/*  $Id: WeakHashtable.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Vector;
/** 
* Observable's observers registration and notification manager.
* Each Observable must have its own ObserverManager to delagate all
* its observers management. The observer manager is published by 
* implementing accessor from Observable interface to let any observer
* register or deregister itself. Then an observable must invoke 
* Notify(this) to its ObserverManager that will relay the call to all
* registered observers.
*/
public class ObserverManager<T> 
{

	/** 
	 * Register an observer.
	 * @param obs observer to register.
	 * @return 
	 *  - true: observer is successfully registered.
	 *  - false: observer not registered (for example if already 
	 * registered)
	 */
	public boolean add(Observer<T> obs) {
	    boolean res=false;
	    synchronized (_observers) {
			if (!_observers.contains(obs)) {
				_observers.add(obs);
				res=true;
			}
	    }
	    return res;
	}
	/** 
	 * Deregister an observer.
	 * @param obs observer to remove
	 * @return 
	 * - true: observer removed. 
	 * - false: observer not removed (for exemple because not present)
	 */
	public boolean remove(Observer<T> obs) {
	    boolean res=false;
	    synchronized (_observers) {
			if (_observers.contains(obs)) {
				_observers.remove(obs);
				res=true;
			}
	    }
	    return res;
	}
	/** 
	 * Notifiy observers.
	 * <p>Cette méthode ne devrait être appelé que par l'observable
	 * dont ce gestionnaire est membre avec pour paramètre l'observable
	 * lui-même.</p>
	 * <p><b>warning:</b> This method is synchronized on the registered observer 
     * list. So observers ObservableChanged() method should notbe too long
	 * to avoid risk of hanging registration from other thread. 
	 * ObservableChanged() implementation should also not invoke 
	 * ObserverManager's add() and remove() methods.</p>
	 * @param sender notification source.
	 */
	public void notify(T sender) {
	    synchronized (_observers) {
			for (Observer<T> obs : _observers) {
			    obs.observableChanged(sender);
			}
	    }
	}
	/** 
	 * Registered observers list. 
	 * @djin.relation aggregation,-,*,Observer&lt;T&gt;
	 */
	private Vector<Observer<T>> _observers=new Vector<Observer<T>>();
}
